<?php

class Referrer {
	
	//---------------------------------------------------------
	//  検索エンジン・検索語解析 関数
	//---------------------------------------------------------
	
	public static function parseWords($db, $referrer) {
		
		// 即時終了フラグ
		$doReturn = false;
		
		// URLパスを取得
		$path = parse_url($referrer, PHP_URL_PATH);
		
		// URLパスが空欄の時（検索語なし）
		if (!$path) {
			$url = $referrer . '/';
			$doReturn = true;
		}
		
		// URLパスが「/」の時（検索語なし）
		elseif ($path === '/') {
			$url = $referrer;
			$doReturn = true;
		}
		
		// 上記以外の時
		else {
			$url = $referrer;
		}
		
		// 即時終了の時
		if ($doReturn) {
			
			// GoogleのURLを統一
			if (strpos($url, 'google') !== false) {
				$url = 'https://www.google.com/';
			}
			
			// リンク元ID、検索語を返す
			return self::parseId($db, $url);
			
		}
		
		// 特殊文字を変換
		$url = preg_replace("/%25/", '%', $url);
		$url = preg_replace("/%2B/", '+', $url);
		
		// URLデコード
		$url = urldecode($url);
		
		// Web用にエンコード
		$url = Logging::toWebStr($url);
		
		// Googleキャッシュの時
		if (preg_match('/q=cache:[^ ]* ([^&]*)&/', $url, $m)) {
			$url = 'https://www.google.com/';
		}
		
		// Googleの時
		elseif (preg_match('/google.*[^\w]q=([^&]*)/', $url, $m)) {
			$url = 'https://www.google.com/';
		}
		
		// Googleの時2
		elseif (preg_match('/google.*(url|imgres)\?/', $url, $m)) {
			$url = 'https://www.google.com/';
			$m[1] = 'not provided';
		}
		
		// Yahooの時1
		elseif (preg_match('/yahoo.*[^\w]p=([^&]*)/', $url, $m)) {
			$url = 'https://search.yahoo.co.jp/';
		}
		
		// Yahooの時2
		elseif (preg_match('/yahoo.*[^\w]RU=/', $url, $m)) {
			$url = 'https://search.yahoo.co.jp/';
		}
		
		// Bingの時
		elseif (preg_match('/bing.*[^\w]q=([^&]*)/', $url, $m)) {
			$url = 'https://www.bing.com/';
		}
		
		// 百度の時
		elseif (preg_match('/baidu.*[^\w]wd=([^&]*)/', $url, $m)) {
			$url = 'https://www.baidu.jp/';
		}
		
		// gooの時
		elseif (preg_match('/goo.*[^\w]MT=([^&]*)/', $url, $m)) {
			$url = 'https://search.goo.ne.jp/';
		}
		
		// infoseekの時
		elseif (preg_match('/infoseek.*[^\w]qt=([^&]*)/', $url, $m)) {
			$url = 'https://www.infoseek.co.jp/';
		}
		
		// exciteの時
		elseif (preg_match('/excite.*[^\w]search=([^&]*)/', $url, $m)) {
			$url = 'https://www.excite.co.jp/';
		}
		
		// はてなの時
		elseif (preg_match('/hatena.*[^\w]word=([^&]*)/', $url, $m)) {
			$url = 'https://search.hatena.ne.jp/';
		}
		
		// 楽天の時
		elseif (preg_match('/rakuten.*[^\w]qt=([^&]*)/', $url, $m)) {
			$url = 'https://www.infoseek.co.jp/';
		}
		
		// @niftyの時
		elseif (preg_match('/nifty.*[^\w]q=([^&]*)/', $url, $m)) {
			$url = 'https://search.nifty.com/';
		}
		
		// @niftyの時2
		elseif (preg_match('/nifty.*[^\w]Text=([^&]*)/', $url, $m)) {
			$url = 'https://search.nifty.com/';
		}
		
		// BIGLOBEの時
		elseif (preg_match('/biglobe.*[^\w]q=([^&]*)/', $url, $m)) {
			$url = 'https://search.biglobe.ne.jp/';
		}
		
		// So-netの時
		elseif (preg_match('/so-net.*[^\w]query=([^&]*)/', $url, $m)) {
			$url = 'https://www.so-net.ne.jp/';
		}
		
		// docomoの時
		elseif (preg_match('/docomo.*[^\w]key=([^&]*)/', $url, $m)) {
			$url = 'https://www.nttdocomo.co.jp/';
		}
		
		// auの時
		elseif (preg_match('/auone.*[^\w]q=([^&]*)/', $url, $m)) {
			$url = 'http://search.auone.jp/';
		}
		
		// auの時2
		elseif (preg_match('/ezweb.*[^\w]query=([^&]*)/', $url, $m)) {
			$url = 'https://www.google.co.jp/';
		}
		
		// conduitの時
		elseif (preg_match('/conduit.*[^\w]q=([^&]*)/', $url, $m)) {
			$url = 'https://search.conduit.com/';
		}
		
		// babylonの時
		elseif (preg_match('/babylon.*[^\w]q=([^&]*)/', $url, $m)) {
			$url = 'https://search.babylon.com/';
		}
		
		// babylonの時2
		elseif (preg_match('/babylon\.com\/web\/([^\?]*)/', $url, $m)) {
			$url = 'https://search.babylon.com/';
		}
		
		// その他検索エンジンの時
		elseif (preg_match('/(.+\?).*(q|query|key|keyword|keywords|MT)=([^&]*)/i', $url, $m)) {
			$url = $m[1] . $m[2] . '=';
			$m[1] = $m[3];
		}
		
		// 検索語をセット
		$searchWords = (isset($m[1])) ? $m[1] : '';
		
		// 検索語が存在する時はスペースを整形
		if ($searchWords) {
			$searchWords = preg_replace('/　|[\s]/', ' ', $searchWords);
			$searchWords = preg_replace('/ +/', ' ', $searchWords);
		}
		
		// リンク元ID、検索語を返す
		return self::parseId($db, $url, $searchWords, true);
		
	}
	
	//---------------------------------------------------------
	//  referrerテーブルの更新
	//---------------------------------------------------------
	
	private static function parseId($db, $url, $searchWords = '', $doRewrite = false) {
		
		// テーブル名を取得
		$refTable = $GLOBALS['config']['table']['referrer'];
		
		// URLをコピー
		$orgUrl = $url;
		
		// 検索語が存在しない時はURLを整形
		if (!$searchWords && $doRewrite) {
			$url = Logging::rewriteUrl($url, true);
		}
		
		// URLをエスケープ
		$url = $db->escape($url);
		
		// クエリーを作成
		$selectIdQuery = "select id from $refTable where url = $url;";
		
		// クエリーを実行
		$id = $db->queryFetch($selectIdQuery, 'id');
		
		// IDが存在する時は終了
		if ($id) {
			return array($id, $searchWords);
		}
		
		// ページタイトルを取得
		$title = Logging::parseTitle($orgUrl);
		
		// タイトルをエスケープ
		$title = $db->escape($title);
		
		////////////////////////////////////////////////////////////
		
		// クエリーを作成
		$selectMaxQuery = "select max(id) as max from $refTable;";
		
		// クエリーを実行
		$max = $db->queryFetch($selectMaxQuery, 'max');
		
		// データが存在する時
		$nextId = ($max) ? $max + 1 : 1;
		
		// クエリーを作成
		$insertQuery = "insert into $refTable (id, url, title) values ($nextId, $url, $title);";
		
		// クエリーを実行
		$db->query($insertQuery);
		
		// リンク元No、検索語を返す
		return array($nextId, $searchWords);
		
	}
	
}

