<?php

class Controller {
	
	//---------------------------------------------------------
	//  メインコントローラ
	//---------------------------------------------------------
	
	public static function execute($baseDir, $workDir) {
		
		// config.iniのパスを設定
		$config = $workDir . '/config.ini';
		
		// config.iniが存在する時
		if (file_exists($config)) {
			
			// システム設定ファイルを解析
			$GLOBALS['config'] = parse_ini_file($config, true);
			
		}
		
		// config.iniが存在しない時
		else {
			
			// default.iniのパスを設定
			$default = $workDir . '/default.ini';
			
			// システム設定ファイルを解析
			$GLOBALS['config'] = parse_ini_file($default, true);
			
			// default.iniをconfig.iniにコピー
			copy($default, $config);
			
		}
		
		////////////////////////////////////////////////////////////
		
		// ディレクトリ＆ファイルパスを定義
		$fwDir = $baseDir . '/framework';
		$dataDir = $baseDir . '/data';
		$errorLog = $dataDir . '/' . $GLOBALS['config']['framework']['errorLog'];
		
		// エラー関連のオプションを設定
		ini_set('error_reporting', E_ALL);
		ini_set('display_errors', $GLOBALS['config']['framework']['displayErrors']);
		ini_set('log_errors', $GLOBALS['config']['framework']['logErrors']);
		ini_set('error_log', $errorLog);
		
		// タイムゾーンを設定
		ini_set('date.timezone', $GLOBALS['config']['framework']['timeZone']);
		
		// ファイルオープンを設定
		ini_set('allow_url_fopen', 1);
		
		////////////////////////////////////////////////////////////
		
		// [URLパスの取得] //
		
		// スクリプトファイルパスを取得
		$root = $_SERVER['SCRIPT_NAME'];
		
		// PATH_INFOがある時は削除
		$root = preg_replace('/(\/.+\.php).*/', "$1", $root);
		
		// スクリプトディレクトリを取得
		$rootDir = dirname($root);
		
		// CGI版でない時はディレクトリをスクリプトパスに設定
		if (strpos(PHP_SAPI, 'cgi') === false && isset($_SERVER['MOD_REWRITE'])) {
			$root = $rootDir;
		}
		
		// パスを定数にセット
		define('BASE_DIR', $baseDir);
		define('WORK_DIR', $workDir);
		define('DATA_DIR', $dataDir);
		define('FW_DIR', $fwDir);
		define('ROOT', $root);
		define('ROOT_DIR', $rootDir);
		define('MAIN_DB', $GLOBALS['config']['framework']['mainDB']);
		
		////////////////////////////////////////////////////////////
		
		// [呼出引数の取得] //
		
		// URL引数
		$pathInfo = '';
		
		// 引数を$GLOBALS['args']にセット
		$GLOBALS['args'] = ($_SERVER['REQUEST_METHOD'] === 'GET') ? $_GET : $_POST;
		
		// ORIG_PATH_INFO が存在する時
		if (isset($_SERVER['ORIG_PATH_INFO'])) {
			
			// 「/」をエスケープ
			$root = addcslashes($root, '/');
			
			// スクリプトファイルパスを削除
			$pathInfo = preg_replace("/$root/A", '', $_SERVER['ORIG_PATH_INFO']);
			
		}
		
		// PATH_INFO が存在する時
		elseif (isset($_SERVER['PATH_INFO'])) {
			
			// PATH_INFO を取得
			$pathInfo = $_SERVER['PATH_INFO'];
			
		}
		
		////////////////////////////////////////////////////////////
		
		// [アクションのディスパッチ] //
		
		// PATH_INFO が存在する時
		if ($pathInfo && $pathInfo !== '/') {
			
			// スラッシュで分割
			$args = explode('/', substr($pathInfo, 1));
			
			// 引数の数を取得
			$end = count($args) - 1;
			
			// $GLOBALS['args']へ追記
			foreach ($args as $arg) {
				$GLOBALS['args'][] = $arg;
			}
			
			// 末尾の空白を削除
			if (!$GLOBALS['args'][$end]) {
				unset($GLOBALS['args'][$end]);
			}
			
			// 第一引数をアクションに設定
			$GLOBALS['args']['act'] = $GLOBALS['args'][0];
			
		}
		
		// PATH_INFO が存在しない時
		else {
			
			// デフォルトのアクションを設定
			if (!isset($GLOBALS['args']['act'])) {
				$GLOBALS['args']['act'] = 'index';
			}
			
		}
		
		// アッパーキャメルに変換
		$class = self::toUpperCamel($GLOBALS['args']['act']);
		
		// アクションが未定義の時
		if (!file_exists($workDir . '/actions/' . $class . '.php')) {
			exit("Undefined Action [$class]");
		}
		
		// クラス名を保持
		$GLOBALS['args']['class'] = $class;
		
		////////////////////////////////////////////////////////////
		
		// [テンプレートクラスのインクルード] //
		
		// テンプレートクラスを読み込み
		include($fwDir . '/modules/core/Template.php');
		
		// インスタンスを生成
		$GLOBALS['object']['tmpl'] = new Template($workDir);
		
		////////////////////////////////////////////////////////////
		
		// [メイン処理] //
		
		// クラスを読み込み
		include($workDir . '/actions/' . $class . '.php');
		
		// コントロール設定を取得
		list($doAuth, $doConnect, $pre, $post) = call_user_func($class . '::control');
		
		// ログイン認証
		if ($doAuth && $GLOBALS['config']['login']['username'] !== '' && $GLOBALS['config']['login']['password'] !== '') {
			
			// ログイン認証クラスを読み込み
			include($fwDir . '/modules/core/Login.php');
			
			// ログイン認証実行
			Login::execute();
			
		}
		
		// DB接続
		if ($doConnect) {
			
			// DBクラスを読み込み
			include($fwDir . '/modules/core/DB.php');
			
			// インスタンスを生成
			$GLOBALS['object']['db'] = new DB($dataDir);
			
			// DBに接続
			$GLOBALS['object']['db']->connect($GLOBALS['config']['framework']['mainDB']);
			
		}
		
		// プレフィルター
		if ($pre) {
			
			// クラスを読み込み
			include($workDir . '/filters/' . $pre . '.php');
			
			// プレ処理を実行
			call_user_func($pre . '::execute');
			
		}
		
		// アクションを実行
		call_user_func($class . '::execute');
		
		// ポストフィルター
		if ($post) {
			
			// クラスを読み込み
			include($workDir . '/filters/' . $post . '.php');
			
			// ポスト処理を実行
			call_user_func($post . '::execute');
			
		}
		
	}
	
	//---------------------------------------------------------
	//  アッパーキャメルに変換
	//---------------------------------------------------------
	
	public static function toUpperCamel($str) {
		
		// 2文字の時
		if (strlen($str) === 2) {
			return strtoupper($str);
		}
		
		// スネークの時
		elseif (strpos($str, '_') !== false) {
			
			// 「_」で分割
			$array = explode('_', $str);
			
			// 変換後の文字列
			$newStr = '';
			
			// 変換して追記
			foreach ($array as $val) {
				$newStr .= ucfirst($val);
			}
			
			// 変換後の文字列を返す
			return $newStr;
			
		}
		
		// 通常文字列の時
		else {
			return ucfirst($str);
		}
		
	}
	
	//---------------------------------------------------------
	//  ローワーキャメルに変換
	//---------------------------------------------------------
	
	public static function toLowerCamel($str) {
		
		// スネークの時
		if (strpos($str, '_') !== false) {
			
			// 「_」で分割
			$array = explode('_', $str);
			
			// 変換後の文字列
			$newStr = array_shift($array);
			
			// 変換して追記
			foreach ($array as $val) {
				$newStr .= ucfirst($val);
			}
			
			// 変換後の文字列を返す
			return $newStr;
			
		}
		
		// 通常文字列の時
		else {
			return $str;
		}
		
	}
	
	//---------------------------------------------------------
	//  時刻を配列で取得
	//---------------------------------------------------------
	
	public static function dates($days = 0) {
		
		// タイムスタンプを設定
		$timestamp = (!$days) ? $_SERVER['REQUEST_TIME'] : strtotime("$days days");
		
		// 現在時刻を返す
		return explode(' ', gmdate('Y m d H i s D', $timestamp + $GLOBALS['config']['framework']['gmtDiff'] * 60 * 60));
		
	}
	
	//---------------------------------------------------------
	//  クラスライブラリを読み込み
	//---------------------------------------------------------
	
	public static function load($name) {
		
		// クラス名を取得
		$class = self::toUpperCamel($name);
		
		// クラスを読み込み
		include_once(FW_DIR . '/modules/library/' . $class . '.php');
		
		// インスタンスを返す
		return new $class();
		
	}
	
}

